#/bin/bash

# --------------------------------------------------------------------------
# File:        backup_database.bash
#
# Autor:       Martin Zahn, Akadia AG
#
# Version:     14.03.2010 / Martin Zahn
#
# Purpose:     Backup Oracle Database using RMAN (Physical Backup)
#              for Primary or Standby Databases. For Primary
#              Databases a DataPump (Logical Backup) is performed
#              as well.
#
# Usage:       backup_database.bash 1>${BACKUP_LOG} 2>&1
#              as a Unix User which belongs to the DBA group
# --------------------------------------------------------------------------

# --------------------------------------------------------------------------
#           ***** Set the following Variables to your need *****
# --------------------------------------------------------------------------

# Keep Backup online for X days
# -----------------------------
BACKUP_WINDOW=1
export BACKUP_WINDOW

# Oracle Environment
# ------------------
ORACLE_ADMIN=/home/oracle/admin
export ORACLE_ADMIN

# Set Location of RMAN Backup (cannot be the same as Location of DataPump Backup)
# -------------------------------------------------------------------------------
RMAN_BACKUP_DEST=/home/oracle/backup/rman
export RMAN_BACKUP_DEST

# Set Location of DataPump Backup (cannot be the same as Location of RMAN Backup)
# -------------------------------------------------------------------------------
DATAPUMP_DEST=/home/oracle/backup/pump
export DATAPUMP_DEST

# --------------------------------------------------------------------------
#      ***** Function to perform Backup for Standby Databases *****
# --------------------------------------------------------------------------
function backup_standby
{
   ${ORACLE_HOME}/bin/rman nocatalog target / <<-STANDBY_BACKUP 1>${BACKUP_LOG} 2>&1
      configure backup optimization on;
      configure controlfile autobackup off;
      configure default device type to disk;
      configure device type disk parallelism 1 backup type to compressed backupset;
      configure datafile backup copies for device type disk to 1;
      configure maxsetsize to unlimited;
      show all;

      run {
        allocate channel ch1 type Disk maxpiecesize = 1900M;
        backup archivelog all
        delete all input
        format '${RMAN_BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_archivelog_%s_%p.bak'
        tag 'archivelog_daily';
      }

      crosscheck backup;
      # restore controlfile validate;
      # restore tablespace SYSTEM validate;
      # restore archivelog all validate;
      list backup of database;
      report unrecoverable;
      report schema;
      report need backup;
      report obsolete;
      delete noprompt expired backup of database;
      delete noprompt expired backup of controlfile;
      delete noprompt expired backup of archivelog all;
      delete noprompt obsolete recovery window of ${BACKUP_WINDOW} days;
      quit
STANDBY_BACKUP
   if [ "$?" != 0 ]
   then
      echo "Error: Oracle RMAN Backup on Host: $(uname -n) for ORACLE_SID: ${ORACLE_SID} failed."
      exit 1
   fi
}

# --------------------------------------------------------------------------
#      ***** Function to perform Backup for Primary Databases *****
# --------------------------------------------------------------------------
# configure retention policy to recovery window of ${BACKUP_WINDOW} days;
function backup_primary
{
   cd ${RMAN_BACKUP_DEST} 1>/dev/null 2>&1
   if [ $(pwd) != ${RMAN_BACKUP_DEST} ]
   then
      MSG=$(echo "Error: Backup Directory: ${RMAN_BACKUP_DEST} not exists ... exiting")
      echo ${MSG}
      exit 1
   fi

   ${ORACLE_HOME}/bin/rman nocatalog target / <<-PRIMARY_BACKUP 1>${BACKUP_LOG} 2>&1
      configure retention policy to redundancy ${BACKUP_WINDOW};
      configure backup optimization on;
      configure controlfile autobackup off;
      configure default device type to disk;
      configure device type disk parallelism 1 backup type to compressed backupset;
      configure datafile backup copies for device type disk to 1;
      configure channel device type Disk format '${RMAN_BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_datafile_%s_%p.bak';
      configure maxsetsize to unlimited;
      configure snapshot controlfile name to '${RMAN_BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_snapshot_controlfile';
      set controlfile autobackup format for device type Disk to '${RMAN_BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_controlfile_%F';
      show all;

      run {
        allocate channel ch1 type Disk maxpiecesize = 1900M;
        backup full database noexclude
        include current controlfile
        format '${RMAN_BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_datafile_%s_%p.bak'
        tag 'datafile_daily';
      }

      run {
        allocate channel ch1 type Disk maxpiecesize = 1900M;
        backup archivelog all
        delete all input
        format '${RMAN_BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_archivelog_%s_%p.bak'
        tag 'archivelog_daily';
      }

      run {
         allocate channel ch1 type Disk maxpiecesize = 1900M;
         backup format '${RMAN_BACKUP_DEST}/${BACKUP_DATE}_${ORACLE_SID}_controlfile_%s.bak' current controlfile;
      }

      crosscheck backup;
      # restore controlfile validate;
      # restore tablespace SYSTEM validate;
      # restore archivelog all validate;
      list backup of database;
      report unrecoverable;
      report schema;
      report need backup;
      report obsolete;
      delete noprompt expired backup of database;
      delete noprompt expired backup of controlfile;
      delete noprompt expired backup of archivelog all;
      delete noprompt obsolete recovery window of ${BACKUP_WINDOW} days;
      quit
PRIMARY_BACKUP
   if [ "$?" != 0 ]
   then
      echo "Error: Oracle RMAN Backup on Host: $(uname -n) for ORACLE_SID: ${ORACLE_SID} failed."
      echo "Trying DataPump Backup ..."
   fi

   # Store the old Password for SYSTEM
   # ---------------------------------
   ${ORACLE_HOME}/bin/sqlplus -s "/ as sysdba" <<-SAVE_PASSWORD 1>/dev/null 2>&1
      set heading off
      set pagesize 0
      set tab off
      set trimspool on
      spool /tmp/old_password.lst
      SELECT password FROM dba_users WHERE UPPER(username) = 'SYSTEM';
      spool off;
SAVE_PASSWORD

   OLD_PASSWOD=$(cat /tmp/old_password.lst | sed -e 's/^$//' -e '/^ *$/d')
   export OLD_PASSWOD
   rm -f /tmp/old_password.lst 1>/dev/null 2>&1

   # Change temporarily the SYSTEM Password
   # --------------------------------------
   ${ORACLE_HOME}/bin/sqlplus -s "/ as sysdba" <<-TMP_PASSWORD 1>/dev/null 2>&1
      ALTER USER SYSTEM IDENTIFIED BY manager;
TMP_PASSWORD

   # Create DataPump Directory
   # -------------------------
   ${ORACLE_HOME}/bin/sqlplus -s "system/manager" <<-DATA_PUMP_DIR 1>/dev/null 2>&1
      CREATE OR REPLACE DIRECTORY ${DATAPUMP_DIR} AS '${DATAPUMP_DEST}';
      exit;
DATA_PUMP_DIR

   # Start the DataPump logical backup
   # ---------------------------------
   cd ${DATAPUMP_DEST}
   if [ $(pwd) != ${DATAPUMP_DEST} ]
   then
      MSG=$(echo "Error: Backup Directory: ${DATAPUMP_DEST} not exists ... exiting")
      echo ${MSG}
      exit 1
   fi

   DATAPUMP_USR_PWD="system/manager";
   export DATAPUMP_USR_PWD
   ${ORACLE_HOME}/bin/expdp userid=${DATAPUMP_USR_PWD} directory=${DATAPUMP_DIR} parallel=3 full=y dumpfile=${DATAPUMP_FILE} logfile=${DATAPUMP_LOG}

   # Compress Datapump Backup and cleanup
   # ------------------------------------
   gzip -f ${DATAPUMP_FILE}
   find . -type f -mtime +5 -exec rm -f {} \;

   # Reset the SYSTEM Password
   # -------------------------
   ${ORACLE_HOME}/bin/sqlplus -s "/ as sysdba" <<-RESET_PASSWORD 1>/dev/null 2>&1
      ALTER USER SYSTEM IDENTIFIED BY VALUES '${OLD_PASSWOD}';
RESET_PASSWORD
}

# --------------------------------------------------------------------------
#                        ***** Start Script *****
# --------------------------------------------------------------------------

# DEBUG
# set -x

# Check that User belongs to the DBA Group
# ----------------------------------------
DBA_GROUP=$(/usr/bin/id -G -n | grep "dba" | wc -l)
if [ "${DBA_GROUP}" != "1" ]
then
   MSG=$(echo "Error: Executing User must be a Member of the DBA Group ... exiting!")
   echo ${MSG}
   exit 1
fi

# Setup ORACLE Environment
# ------------------------
ORACLE_SID=$1
if [ -z "${ORACLE_SID}" ]
then
   MSG=$(echo "Usage: $0: ORACLE_SID ... exiting!")
   echo ${MSG}
   exit 1
fi
if [ ! -f ${ORACLE_ADMIN}/${ORACLE_SID}/.oraenv ]
then
   MSG=$(echo "Error: Couldn't find ${ORACLE_ADMIN}/${ORACLE_SID}/.oraenv to set Oracle Environment ... exiting")
   echo ${MSG}
   exit 1
else
   .  ${ORACLE_ADMIN}/${ORACLE_SID}/.oraenv
fi

# Set and Check Backup Destination
# --------------------------------
BACKUP_DATE=$(date +%Y%m%d); export BACKUP_DATE
cd ${RMAN_BACKUP_DEST} 1>/dev/null 2>&1
if [ $(pwd) != ${RMAN_BACKUP_DEST} ]
then
   MSG=$(echo "Error: Backup Directory: ${RMAN_BACKUP_DEST} not exists ... exiting")
   echo ${MSG}
   exit 1
fi

cd ${DATAPUMP_DEST} 1>/dev/null 2>&1
if [ $(pwd) != ${DATAPUMP_DEST} ]
then
   MSG=$(echo "Error: Backup Directory: ${DATAPUMP_DEST} not exists ... exiting")
   echo ${MSG}
   exit 1
fi

if [ "${RMAN_BACKUP_DEST}" = "${DATAPUMP_DEST}" ]
then
   MSG=$(echo "Error: RMAN Backup Directory: ${RMAN_BACKUP_DEST} cannot be the same as the DataPump Backup Directory: ${DATAPUMP_DEST}")
   echo ${MSG}
   exit 1
fi

# Set DataPump Parameters
# -----------------------
DATAPUMP_FILE=${BACKUP_DATE}_datapump_full_${ORACLE_SID}.dp
export DATAPUMP_FILE

DATAPUMP_LOG=${BACKUP_DATE}_datapump_full_${ORACLE_SID}.log
export DATAPUMP_LOG

DATAPUMP_DIR=dp${ORACLE_SID};
export DATAPUMP_DIR

# Logfile
# -------
BACKUP_LOG=${RMAN_BACKUP_DEST}/${BACKUP_DATE}_backup_database.log
export BACKUP_LOG

# Check ARCHIVELOG Mode
# ---------------------
${ORACLE_HOME}/bin/sqlplus -s "/ as sysdba" <<-ARCHIVELOG 1>/dev/null 2>&1
   set feed off;
   set wrap on;
   set heading off;
   set tab off;
   set trimspool on;
   spool archivelog.log
   ARCHIVE LOG LIST;
   spool off;
   exit;
ARCHIVELOG
ARCHIVELOG_CHECK=$(cat archivelog.log | grep "Enabled")
if [ -z "${ARCHIVELOG_CHECK}" ]
then
   MSG=$(echo "Error: Unable to connect to: ${ORACLE_SID} or not in ARCHIVELOG Mode ... exiting!")
   echo ${MSG}
   rm -f archivelog.log 1>/dev/null 2>&1
   exit 1
fi
rm -f archivelog.log 1>/dev/null 2>&1

# Get Database Role
# -----------------
${ORACLE_HOME}/bin/sqlplus -s "/ as sysdba" <<-DATABASE_ROLE 1>/dev/null 2>&1
   set feed off;
   set wrap on;
   set heading off;
   set tab off;
   set trimspool on;
   spool database_role.log
   SELECT database_role FROM v\$database;
   spool off;
   exit;
DATABASE_ROLE

# Get Database Role
# -----------------
STANDBY_ROLE="STANDBY"
PRIMARY_ROLE="PRIMARY"
if [ -s database_role.log ]
then
   DATABASE_ROLE=$(cat database_role.log | grep "STANDBY")
   if [ ! -z "${DATABASE_ROLE}" ]
   then
      DATABASE_ROLE=${STANDBY_ROLE}
   else
      DATABASE_ROLE=${PRIMARY_ROLE}
   fi
else
   MSG=$(echo "Error: Database Role could not be verified ... exiting!")
   echo ${MSG}
   rm -f database_role.log 1>/dev/null 2>&1
fi
rm -f database_role.log 1>/dev/null 2>&1

# Backup Parameterfiles
# ---------------------
if [ -f ${ORACLE_HOME}/dbs/init${ORACLE_SID}.ora ]
then
   cp ${ORACLE_HOME}/dbs/init${ORACLE_SID}.ora ${RMAN_BACKUP_DEST}/${BACKUP_DATE}_init${ORACLE_SID}.ora
fi
if [ -f ${ORACLE_HOME}/dbs/spfile${ORACLE_SID}.ora ]
then
   cp ${ORACLE_HOME}/dbs/spfile${ORACLE_SID}.ora ${RMAN_BACKUP_DEST}/${BACKUP_DATE}_spfile${ORACLE_SID}.ora
fi

# Backup Passwordfile
# -------------------
if [ -f ${ORACLE_HOME}/dbs/orapw${ORACLE_SID} ]
then
   cp ${ORACLE_HOME}/dbs/orapw${ORACLE_SID} ${RMAN_BACKUP_DEST}/${BACKUP_DATE}_orapw${ORACLE_SID}
fi

# Backup Trace and Audit Files
# ----------------------------
rm -f user_dump_dest.log 1>/dev/null 2>&1
${ORACLE_HOME}/bin/sqlplus -s "/ as sysdba" <<-GET_USER_DUMP 1>/dev/null 2>&1
   set feed off;
   set wrap on;
   set heading off;
   set tab off;
   set trimspool on;
   spool user_dump_dest.log
   SELECT value FROM v\$parameter WHERE LOWER(name) = 'user_dump_dest';
   spool off;
   exit;
GET_USER_DUMP
if [ -s user_dump_dest.log ]
then
   USER_DUMP_DEST=$(tail -1 user_dump_dest.log)
   ORA_LOG_DIR=$(dirname ${USER_DUMP_DEST})
fi
rm -f user_dump_dest.log 1>/dev/null 2>&1
cd ${ORA_LOG_DIR} 1>/dev/null 2>&1
if [ "$?" != "0" ]
then
   MSG=$(echo "Error: Log Directory ${ORA_LOG_DIR} doesn't exist ... exiting!")
   echo ${MSG}
   exit 1
fi
if [ "$(pwd)" = "${ORA_LOG_DIR}" ]
then
   TRACE_FILES=$(find . -print)
   if [ ! -z "${TRACE_FILES}" ]
   then
     tar czf ${RMAN_BACKUP_DEST}/${BACKUP_DATE}_trace_audit.tgz ${TRACE_FILES}
     find . -type f -exec rm {} \; 1>/dev/null 2>&1
   fi
fi

# Start the Database Backup
# -------------------------
echo "Database Backup on Host: $(uname -n) for ORACLE_SID: ${ORACLE_SID} for Role: ${DATABASE_ROLE} started."
if [ "${DATABASE_ROLE}" = "${STANDBY_ROLE}" ]
then
   backup_standby
else
   backup_primary
fi

# Create TAR/GZIP file with all the backuped files
# ------------------------------------------------
cd ${RMAN_BACKUP_DEST}
if [ "$(pwd)" = "${RMAN_BACKUP_DEST}" ]
then
   BACKUPSET_RMAN="${BACKUP_DATE}_backupset_rman_${ORACLE_SID}.tgz"
   tar czvf ../${BACKUPSET_RMAN} ${BACKUP_DATE}_*
   mv ../${BACKUPSET_RMAN} .
   find . -type f -mtime +${BACKUP_WINDOW} -exec rm -f {} \;
fi

cd ${DATAPUMP_DEST}
if [ "$(pwd)" = "${DATAPUMP_DEST}" ]
then
   BACKUPSET_DATAPUMP="${BACKUP_DATE}_backupset_datapump_${ORACLE_SID}.tgz"
   tar czvf ../${BACKUPSET_DATAPUMP} ${BACKUP_DATE}_*
   mv ../${BACKUPSET_DATAPUMP} .
   find . -type f -mtime +${BACKUP_WINDOW} -exec rm -f {} \;
fi

echo "Database Backup on Host: $(uname -n) for ORACLE_SID: ${ORACLE_SID} for Role: ${DATABASE_ROLE} finished."
exit 0